/****************************************************************************
** Meta object code from reading C++ file 'mainframe.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../include/ui/mainFrame/mainframe.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainframe.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9MainFrameE_t {};
} // unnamed namespace

template <> constexpr inline auto MainFrame::qt_create_metaobjectdata<qt_meta_tag_ZN9MainFrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainFrame",
        "isHeaderChecked",
        "",
        "checked",
        "tableChanged",
        "index",
        "redownload",
        "taskId",
        "rd",
        "ariaOption",
        "isHasTracker",
        "isHasDHT",
        "saveNameBeforChangeList",
        "OpenFile",
        "url",
        "Raise",
        "onTrayQuitClick",
        "force",
        "onReceiveExtentionUrl",
        "onActivated",
        "QSystemTrayIcon::ActivationReason",
        "reason",
        "onSettingsMenuClicked",
        "onRpcSuccess",
        "method",
        "QJsonObject",
        "json",
        "onRpcError",
        "id",
        "error",
        "obj",
        "onTableItemSelected",
        "QModelIndex",
        "selected",
        "onClipboardDataChanged",
        "onListClicked",
        "onPalettetypechanged",
        "DGuiApplicationHelper::ColorType",
        "type",
        "onDownloadNewUrl",
        "savePath",
        "fileName",
        "leng",
        "onDownloadNewTorrent",
        "btPath",
        "QMap<QString,QVariant>&",
        "opt",
        "infoName",
        "infoHash",
        "onDownloadNewMetalink",
        "linkPath",
        "onHeaderStatechanged",
        "isChecked",
        "onContextMenu",
        "QPoint",
        "pos",
        "onSearchEditTextChanged",
        "text",
        "onUpdateMainUI",
        "onNewBtnClicked",
        "onStartDownloadBtnClicked",
        "onPauseDownloadBtnClicked",
        "onDownloadFirstBtnClicked",
        "onDeleteDownloadBtnClicked",
        "onCheckChanged",
        "flag",
        "onDeleteConfirm",
        "ischecked",
        "permanent",
        "onRedownload",
        "onRemoveFinished",
        "onAria2Remove",
        "gId",
        "onMessageBoxConfirmClick",
        "onDeleteActionTriggered",
        "onRedownloadActionTriggered",
        "onReturnOriginActionTriggered",
        "onOpenFileActionTriggered",
        "onOpenFolderActionTriggered",
        "onRenameActionTriggered",
        "onMoveToActionTriggered",
        "onClearRecyleActionTriggered",
        "onCopyUrlActionTriggered",
        "onDeletePermanentActionTriggered",
        "onIsStartAssociatedBTFile",
        "status",
        "onIsControlBrowser",
        "onAutoDownloadBySpeed",
        "onTableViewItemDoubleClicked",
        "onDownloadLimitChanged",
        "onPowerOnChanged",
        "isPowerOn",
        "onMaxDownloadTaskNumberChanged",
        "nTaskNumber",
        "isStopTask",
        "isAddOne",
        "onDisckCacheChanged",
        "nNum",
        "onClearRecycle",
        "onParseUrlList",
        "QList<LinkInfo>&",
        "urlList",
        "path",
        "onDownloadFinish",
        "onSearchItemClicked",
        "QListWidgetItem*",
        "item",
        "continueDownload",
        "Global::DownloadDataItem*",
        "pItem"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'isHeaderChecked'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'tableChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'redownload'
        QtMocHelpers::SignalData<void(QString, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'ariaOption'
        QtMocHelpers::SignalData<void(bool, bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 }, { QMetaType::Bool, 11 },
        }}),
        // Signal 'saveNameBeforChangeList'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'OpenFile'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'Raise'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTrayQuitClick'
        QtMocHelpers::SlotData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Slot 'onTrayQuitClick'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onReceiveExtentionUrl'
        QtMocHelpers::SlotData<void(QString)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'onActivated'
        QtMocHelpers::SlotData<void(QSystemTrayIcon::ActivationReason)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'onSettingsMenuClicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRpcSuccess'
        QtMocHelpers::SlotData<void(QString, QJsonObject)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { 0x80000000 | 25, 26 },
        }}),
        // Slot 'onRpcError'
        QtMocHelpers::SlotData<void(QString, QString, int, QJsonObject)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 24 }, { QMetaType::QString, 28 }, { QMetaType::Int, 29 }, { 0x80000000 | 25, 30 },
        }}),
        // Slot 'onTableItemSelected'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 33 },
        }}),
        // Slot 'onClipboardDataChanged'
        QtMocHelpers::SlotData<void(QString)>(34, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 14 },
        }}),
        // Slot 'onListClicked'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 5 },
        }}),
        // Slot 'onPalettetypechanged'
        QtMocHelpers::SlotData<void(DGuiApplicationHelper::ColorType)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Slot 'onDownloadNewUrl'
        QtMocHelpers::SlotData<void(QString, QString, QString, QString, QString)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 40 }, { QMetaType::QString, 41 }, { QMetaType::QString, 38 },
            { QMetaType::QString, 42 },
        }}),
        // Slot 'onDownloadNewUrl'
        QtMocHelpers::SlotData<void(QString, QString, QString, QString)>(39, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 40 }, { QMetaType::QString, 41 }, { QMetaType::QString, 38 },
        }}),
        // Slot 'onDownloadNewUrl'
        QtMocHelpers::SlotData<void(QString, QString, QString)>(39, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 14 }, { QMetaType::QString, 40 }, { QMetaType::QString, 41 },
        }}),
        // Slot 'onDownloadNewTorrent'
        QtMocHelpers::SlotData<bool(QString, QMap<QString,QVariant> &, QString, QString)>(43, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 44 }, { 0x80000000 | 45, 46 }, { QMetaType::QString, 47 }, { QMetaType::QString, 48 },
        }}),
        // Slot 'onDownloadNewMetalink'
        QtMocHelpers::SlotData<bool(QString, QMap<QString,QVariant> &, QString)>(49, 2, QMC::AccessPrivate, QMetaType::Bool, {{
            { QMetaType::QString, 50 }, { 0x80000000 | 45, 46 }, { QMetaType::QString, 47 },
        }}),
        // Slot 'onHeaderStatechanged'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 52 },
        }}),
        // Slot 'onContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(53, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 54, 55 },
        }}),
        // Slot 'onSearchEditTextChanged'
        QtMocHelpers::SlotData<void(QString)>(56, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 57 },
        }}),
        // Slot 'onUpdateMainUI'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onNewBtnClicked'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onStartDownloadBtnClicked'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPauseDownloadBtnClicked'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDownloadFirstBtnClicked'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDeleteDownloadBtnClicked'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCheckChanged'
        QtMocHelpers::SlotData<void(bool, int)>(64, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 }, { QMetaType::Int, 65 },
        }}),
        // Slot 'onDeleteConfirm'
        QtMocHelpers::SlotData<void(bool, bool)>(66, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 67 }, { QMetaType::Bool, 68 },
        }}),
        // Slot 'onRedownload'
        QtMocHelpers::SlotData<void(QString, int)>(69, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
        }}),
        // Slot 'onRemoveFinished'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onAria2Remove'
        QtMocHelpers::SlotData<void(QString, QString)>(71, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 72 }, { QMetaType::QString, 28 },
        }}),
        // Slot 'onMessageBoxConfirmClick'
        QtMocHelpers::SlotData<void(int)>(73, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'onDeleteActionTriggered'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRedownloadActionTriggered'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onReturnOriginActionTriggered'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onOpenFileActionTriggered'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onOpenFolderActionTriggered'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onRenameActionTriggered'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMoveToActionTriggered'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onClearRecyleActionTriggered'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCopyUrlActionTriggered'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDeletePermanentActionTriggered'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onIsStartAssociatedBTFile'
        QtMocHelpers::SlotData<void(bool)>(84, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 85 },
        }}),
        // Slot 'onIsControlBrowser'
        QtMocHelpers::SlotData<void(bool)>(86, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 85 },
        }}),
        // Slot 'onAutoDownloadBySpeed'
        QtMocHelpers::SlotData<void(bool)>(87, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 85 },
        }}),
        // Slot 'onTableViewItemDoubleClicked'
        QtMocHelpers::SlotData<void(QModelIndex)>(88, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 32, 5 },
        }}),
        // Slot 'onDownloadLimitChanged'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPowerOnChanged'
        QtMocHelpers::SlotData<void(bool)>(90, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 91 },
        }}),
        // Slot 'onMaxDownloadTaskNumberChanged'
        QtMocHelpers::SlotData<void(int, bool, bool)>(92, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 93 }, { QMetaType::Bool, 94 }, { QMetaType::Bool, 95 },
        }}),
        // Slot 'onMaxDownloadTaskNumberChanged'
        QtMocHelpers::SlotData<void(int, bool)>(92, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 93 }, { QMetaType::Bool, 94 },
        }}),
        // Slot 'onMaxDownloadTaskNumberChanged'
        QtMocHelpers::SlotData<void(int)>(92, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 93 },
        }}),
        // Slot 'onDisckCacheChanged'
        QtMocHelpers::SlotData<void(int)>(96, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 97 },
        }}),
        // Slot 'onClearRecycle'
        QtMocHelpers::SlotData<void(bool)>(98, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 67 },
        }}),
        // Slot 'onParseUrlList'
        QtMocHelpers::SlotData<void(QVector<LinkInfo> &, QString)>(99, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 100, 101 }, { QMetaType::QString, 102 },
        }}),
        // Slot 'onDownloadFinish'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSearchItemClicked'
        QtMocHelpers::SlotData<void(QListWidgetItem *)>(104, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 105, 106 },
        }}),
        // Slot 'continueDownload'
        QtMocHelpers::SlotData<void(Global::DownloadDataItem *)>(107, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 108, 109 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainFrame, qt_meta_tag_ZN9MainFrameE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainFrame::staticMetaObject = { {
    QMetaObject::SuperData::link<Dtk::Widget::DMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MainFrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MainFrameE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9MainFrameE_t>.metaTypes,
    nullptr
} };

void MainFrame::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainFrame *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->isHeaderChecked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->tableChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->redownload((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->ariaOption((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->saveNameBeforChangeList(); break;
        case 5: _t->OpenFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->Raise(); break;
        case 7: _t->onTrayQuitClick((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->onTrayQuitClick(); break;
        case 9: _t->onReceiveExtentionUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->onActivated((*reinterpret_cast<std::add_pointer_t<QSystemTrayIcon::ActivationReason>>(_a[1]))); break;
        case 11: _t->onSettingsMenuClicked(); break;
        case 12: _t->onRpcSuccess((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QJsonObject>>(_a[2]))); break;
        case 13: _t->onRpcError((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QJsonObject>>(_a[4]))); break;
        case 14: _t->onTableItemSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 15: _t->onClipboardDataChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->onListClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 17: _t->onPalettetypechanged((*reinterpret_cast<std::add_pointer_t<DGuiApplicationHelper::ColorType>>(_a[1]))); break;
        case 18: _t->onDownloadNewUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 19: _t->onDownloadNewUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 20: _t->onDownloadNewUrl((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 21: { bool _r = _t->onDownloadNewTorrent((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QVariant>&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->onDownloadNewMetalink((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QVariant>&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: _t->onHeaderStatechanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->onContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 25: _t->onSearchEditTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 26: _t->onUpdateMainUI(); break;
        case 27: _t->onNewBtnClicked(); break;
        case 28: _t->onStartDownloadBtnClicked(); break;
        case 29: _t->onPauseDownloadBtnClicked(); break;
        case 30: _t->onDownloadFirstBtnClicked(); break;
        case 31: _t->onDeleteDownloadBtnClicked(); break;
        case 32: _t->onCheckChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 33: _t->onDeleteConfirm((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 34: _t->onRedownload((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 35: _t->onRemoveFinished(); break;
        case 36: _t->onAria2Remove((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 37: _t->onMessageBoxConfirmClick((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 38: _t->onDeleteActionTriggered(); break;
        case 39: _t->onRedownloadActionTriggered(); break;
        case 40: _t->onReturnOriginActionTriggered(); break;
        case 41: _t->onOpenFileActionTriggered(); break;
        case 42: _t->onOpenFolderActionTriggered(); break;
        case 43: _t->onRenameActionTriggered(); break;
        case 44: _t->onMoveToActionTriggered(); break;
        case 45: _t->onClearRecyleActionTriggered(); break;
        case 46: _t->onCopyUrlActionTriggered(); break;
        case 47: _t->onDeletePermanentActionTriggered(); break;
        case 48: _t->onIsStartAssociatedBTFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 49: _t->onIsControlBrowser((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 50: _t->onAutoDownloadBySpeed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 51: _t->onTableViewItemDoubleClicked((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 52: _t->onDownloadLimitChanged(); break;
        case 53: _t->onPowerOnChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 54: _t->onMaxDownloadTaskNumberChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 55: _t->onMaxDownloadTaskNumberChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 56: _t->onMaxDownloadTaskNumberChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 57: _t->onDisckCacheChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 58: _t->onClearRecycle((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 59: _t->onParseUrlList((*reinterpret_cast<std::add_pointer_t<QList<LinkInfo>&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 60: _t->onDownloadFinish(); break;
        case 61: _t->onSearchItemClicked((*reinterpret_cast<std::add_pointer_t<QListWidgetItem*>>(_a[1]))); break;
        case 62: _t->continueDownload((*reinterpret_cast<std::add_pointer_t<Global::DownloadDataItem*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainFrame::*)(bool )>(_a, &MainFrame::isHeaderChecked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainFrame::*)(int )>(_a, &MainFrame::tableChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainFrame::*)(QString , int )>(_a, &MainFrame::redownload, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainFrame::*)(bool , bool )>(_a, &MainFrame::ariaOption, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainFrame::*)()>(_a, &MainFrame::saveNameBeforChangeList, 4))
            return;
    }
}

const QMetaObject *MainFrame::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainFrame::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9MainFrameE_t>.strings))
        return static_cast<void*>(this);
    return Dtk::Widget::DMainWindow::qt_metacast(_clname);
}

int MainFrame::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Dtk::Widget::DMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 63)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 63;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 63)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 63;
    }
    return _id;
}

// SIGNAL 0
void MainFrame::isHeaderChecked(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MainFrame::tableChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MainFrame::redownload(QString _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void MainFrame::ariaOption(bool _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void MainFrame::saveNameBeforChangeList()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
