/****************************************************************************
** Meta object code from reading C++ file 'tabledatacontrol.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../include/ui/mainFrame/tabledatacontrol.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabledatacontrol.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16TableDataControlE_t {};
} // unnamed namespace

template <> constexpr inline auto TableDataControl::qt_create_metaobjectdata<qt_meta_tag_ZN16TableDataControlE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TableDataControl",
        "AutoDownloadBt",
        "",
        "btFilePath",
        "AutoDownloadMetalink",
        "filePath",
        "RedownloadJob",
        "taskId",
        "rd",
        "DownloadUnusuaHttpJob",
        "urlList",
        "savePath",
        "filename",
        "type",
        "leng",
        "DownloadUnusuaBtJob",
        "btPath",
        "QMap<QString,QVariant>&",
        "opt",
        "infoName",
        "infoHash",
        "removeFinished",
        "startDownload",
        "whenDownloadFinish",
        "addMaxDownloadTask",
        "num",
        "unPauseTask",
        "Global::DownloadDataItem*",
        "pItem",
        "onUnusualConfirm",
        "index",
        "taskIds"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'AutoDownloadBt'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'AutoDownloadMetalink'
        QtMocHelpers::SignalData<void(QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'RedownloadJob'
        QtMocHelpers::SignalData<void(QString, int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { QMetaType::Int, 8 },
        }}),
        // Signal 'DownloadUnusuaHttpJob'
        QtMocHelpers::SignalData<void(QString, QString, QString, QString, QString)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
            { QMetaType::QString, 14 },
        }}),
        // Signal 'DownloadUnusuaHttpJob'
        QtMocHelpers::SignalData<void(QString, QString, QString, QString)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 12 }, { QMetaType::QString, 13 },
        }}),
        // Signal 'DownloadUnusuaHttpJob'
        QtMocHelpers::SignalData<void(QString, QString, QString)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 }, { QMetaType::QString, 12 },
        }}),
        // Signal 'DownloadUnusuaHttpJob'
        QtMocHelpers::SignalData<void(QString, QString)>(9, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'DownloadUnusuaBtJob'
        QtMocHelpers::SignalData<void(QString, QMap<QString,QVariant> &, QString, QString)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { 0x80000000 | 17, 18 }, { QMetaType::QString, 19 }, { QMetaType::QString, 20 },
        }}),
        // Signal 'removeFinished'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startDownload'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'whenDownloadFinish'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addMaxDownloadTask'
        QtMocHelpers::SignalData<void(int)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 25 },
        }}),
        // Signal 'unPauseTask'
        QtMocHelpers::SignalData<void(Global::DownloadDataItem *)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Slot 'onUnusualConfirm'
        QtMocHelpers::SlotData<void(int, const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 30 }, { QMetaType::QString, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TableDataControl, qt_meta_tag_ZN16TableDataControlE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TableDataControl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TableDataControlE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TableDataControlE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16TableDataControlE_t>.metaTypes,
    nullptr
} };

void TableDataControl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TableDataControl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->AutoDownloadBt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->AutoDownloadMetalink((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->RedownloadJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 3: _t->DownloadUnusuaHttpJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[5]))); break;
        case 4: _t->DownloadUnusuaHttpJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 5: _t->DownloadUnusuaHttpJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 6: _t->DownloadUnusuaHttpJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 7: _t->DownloadUnusuaBtJob((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QVariant>&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 8: _t->removeFinished(); break;
        case 9: _t->startDownload(); break;
        case 10: _t->whenDownloadFinish(); break;
        case 11: _t->addMaxDownloadTask((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 12: _t->unPauseTask((*reinterpret_cast<std::add_pointer_t<Global::DownloadDataItem*>>(_a[1]))); break;
        case 13: _t->onUnusualConfirm((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(QString )>(_a, &TableDataControl::AutoDownloadBt, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(QString )>(_a, &TableDataControl::AutoDownloadMetalink, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(QString , int )>(_a, &TableDataControl::RedownloadJob, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(QString , QString , QString , QString , QString )>(_a, &TableDataControl::DownloadUnusuaHttpJob, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(QString , QMap<QString,QVariant> & , QString , QString )>(_a, &TableDataControl::DownloadUnusuaBtJob, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)()>(_a, &TableDataControl::removeFinished, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)()>(_a, &TableDataControl::startDownload, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)()>(_a, &TableDataControl::whenDownloadFinish, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(int )>(_a, &TableDataControl::addMaxDownloadTask, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (TableDataControl::*)(Global::DownloadDataItem * )>(_a, &TableDataControl::unPauseTask, 12))
            return;
    }
}

const QMetaObject *TableDataControl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TableDataControl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16TableDataControlE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TableDataControl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void TableDataControl::AutoDownloadBt(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TableDataControl::AutoDownloadMetalink(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TableDataControl::RedownloadJob(QString _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void TableDataControl::DownloadUnusuaHttpJob(QString _t1, QString _t2, QString _t3, QString _t4, QString _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 7
void TableDataControl::DownloadUnusuaBtJob(QString _t1, QMap<QString,QVariant> & _t2, QString _t3, QString _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 8
void TableDataControl::removeFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void TableDataControl::startDownload()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void TableDataControl::whenDownloadFinish()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void TableDataControl::addMaxDownloadTask(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void TableDataControl::unPauseTask(Global::DownloadDataItem * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}
QT_WARNING_POP
