/****************************************************************************
** Meta object code from reading C++ file 'createtaskwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../../../include/ui/createTask/createtaskwidget.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'createtaskwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16CreateTaskWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto CreateTaskWidget::qt_create_metaobjectdata<qt_meta_tag_ZN16CreateTaskWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CreateTaskWidget",
        "downloadWidgetCreate",
        "",
        "QList<LinkInfo>&",
        "urlList",
        "savePath",
        "downLoadTorrentCreate",
        "btPath",
        "QMap<QString,QVariant>&",
        "opt",
        "infoName",
        "infoHash",
        "downLoadMetaLinkCreate",
        "MetaLinkPath",
        "requestProcessBtFile",
        "filePath",
        "isMetalink",
        "onFileDialogOpen",
        "onMLFileDialogOpen",
        "onCancelBtnClicked",
        "onSureBtnClicked",
        "onTextChanged",
        "onFilechoosed",
        "updataTabel",
        "LinkInfo*",
        "onAllCheck",
        "onVideoCheck",
        "onAudioCheck",
        "onPictureCheck",
        "onZipCheck",
        "onDocCheck",
        "onOtherCheck",
        "onProcessBtFileAsync"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'downloadWidgetCreate'
        QtMocHelpers::SignalData<void(QVector<LinkInfo> &, QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'downLoadTorrentCreate'
        QtMocHelpers::SignalData<void(QString, QMap<QString,QVariant> &, QString, QString)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 }, { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Signal 'downLoadMetaLinkCreate'
        QtMocHelpers::SignalData<void(QString, QMap<QString,QVariant> &, QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 }, { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 },
        }}),
        // Signal 'requestProcessBtFile'
        QtMocHelpers::SignalData<void(QString, bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 16 },
        }}),
        // Slot 'onFileDialogOpen'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onMLFileDialogOpen'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onCancelBtnClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSureBtnClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onTextChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onFilechoosed'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'updataTabel'
        QtMocHelpers::SlotData<void(LinkInfo *)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 24, 2 },
        }}),
        // Slot 'onAllCheck'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onVideoCheck'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onAudioCheck'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onPictureCheck'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onZipCheck'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDocCheck'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onOtherCheck'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onProcessBtFileAsync'
        QtMocHelpers::SlotData<void(QString, bool)>(32, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 15 }, { QMetaType::Bool, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CreateTaskWidget, qt_meta_tag_ZN16CreateTaskWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CreateTaskWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<DDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CreateTaskWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CreateTaskWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16CreateTaskWidgetE_t>.metaTypes,
    nullptr
} };

void CreateTaskWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CreateTaskWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->downloadWidgetCreate((*reinterpret_cast<std::add_pointer_t<QList<LinkInfo>&>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->downLoadTorrentCreate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QVariant>&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4]))); break;
        case 2: _t->downLoadMetaLinkCreate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QMap<QString,QVariant>&>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 3: _t->requestProcessBtFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 4: _t->onFileDialogOpen(); break;
        case 5: _t->onMLFileDialogOpen(); break;
        case 6: _t->onCancelBtnClicked(); break;
        case 7: _t->onSureBtnClicked(); break;
        case 8: _t->onTextChanged(); break;
        case 9: _t->onFilechoosed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->updataTabel((*reinterpret_cast<std::add_pointer_t<LinkInfo*>>(_a[1]))); break;
        case 11: _t->onAllCheck(); break;
        case 12: _t->onVideoCheck(); break;
        case 13: _t->onAudioCheck(); break;
        case 14: _t->onPictureCheck(); break;
        case 15: _t->onZipCheck(); break;
        case 16: _t->onDocCheck(); break;
        case 17: _t->onOtherCheck(); break;
        case 18: _t->onProcessBtFileAsync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CreateTaskWidget::*)(QVector<LinkInfo> & , QString )>(_a, &CreateTaskWidget::downloadWidgetCreate, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CreateTaskWidget::*)(QString , QMap<QString,QVariant> & , QString , QString )>(_a, &CreateTaskWidget::downLoadTorrentCreate, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CreateTaskWidget::*)(QString , QMap<QString,QVariant> & , QString )>(_a, &CreateTaskWidget::downLoadMetaLinkCreate, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CreateTaskWidget::*)(QString , bool )>(_a, &CreateTaskWidget::requestProcessBtFile, 3))
            return;
    }
}

const QMetaObject *CreateTaskWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CreateTaskWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CreateTaskWidgetE_t>.strings))
        return static_cast<void*>(this);
    return DDialog::qt_metacast(_clname);
}

int CreateTaskWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void CreateTaskWidget::downloadWidgetCreate(QVector<LinkInfo> & _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void CreateTaskWidget::downLoadTorrentCreate(QString _t1, QMap<QString,QVariant> & _t2, QString _t3, QString _t4)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3, _t4);
}

// SIGNAL 2
void CreateTaskWidget::downLoadMetaLinkCreate(QString _t1, QMap<QString,QVariant> & _t2, QString _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}

// SIGNAL 3
void CreateTaskWidget::requestProcessBtFile(QString _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}
QT_WARNING_POP
